@@ -13,12 +13,14 @@ class Agent(object):
         self.__aesd = lambda k, v: self.__check() and AESCipher(
             EasyRSA(private_key=key_pair["private_key"]).decrypt(k)
         ).decrypt(v)
-        self.__hmac = lambda v: self.__check() and mac(key_pair["public_key"], v)
+        self.__sign = lambda m: EasyRSA(private_key=key_pair["private_key"]).sign(m)
+        self.__verify = lambda m, s: EasyRSA(public_key=key_pair["public_key"]).verify(m, s)
         self.__setk = lambda k, v: self.__check() and mac(sha512(k), v)
-        self.__app_name = self.__hmac(app_name)
+        self.__setn = lambda v: self.__check() and self.__setk(key_pair["public_key"], v)
+        self.__app_name = self.__setn(app_name)
 
     @staticmethod
-    def __check():
+    def __check() -> bool:
         import inspect
         if not inspect.stack()[2][1].replace("\\", ".").replace("/", ".").endswith("site-packages.credsafe.agent.py"):
             raise Exception("call outside Agent() is prohibited")
@@ -28,28 +30,28 @@ class Agent(object):
         self.__check()
         sk, v = self.__aese(jd(v))
         sk = b64e(sk)
-        hash = self.__hmac(v)
+        hash = b64e(self.__sign(v))
         return f"{hash} {sk} {v}"
 
     def set(self, id: str, pw: str, k: str, v: Any) -> Any:
-        Broker(app_name=self.__app_name, username=self.__hmac(id)).set(self.__setk(pw, k), self.__encrypt(v))
+        Broker(app_name=self.__app_name, username=self.__setn(id)).set(self.__setk(pw, k), self.__encrypt(v))
         return self
 
     def __decrypt(self, v: str) -> Any:
         self.__check()
         hash, sk, v = v.split(" ")
-        if hash == self.__hmac(v):
+        if self.__verify(v, hash):
             return jl(self.__aesd(b64d(sk), v))
         raise Exception("credentials are tampered due to different hmac")
 
     def get(self, id: str, pw: str, k: str) -> Any:
-        return self.__decrypt(Broker(app_name=self.__app_name, username=self.__hmac(id)).get(self.__setk(pw, k)))
+        return self.__decrypt(Broker(app_name=self.__app_name, username=self.__setn(id)).get(self.__setk(pw, k)))
 
     def rm(self, id: str, pw: str, k: str) -> bool:
-        return Broker(app_name=self.__app_name, username=self.__hmac(id)).rm(self.__setk(pw, k))
+        return Broker(app_name=self.__app_name, username=self.__setn(id)).rm(self.__setk(pw, k))
 
     def destroy(self, id: str) -> bool:
-        return Broker(app_name=self.__app_name, username=self.__hmac(id)).destroy()
+        return Broker(app_name=self.__app_name, username=self.__setn(id)).destroy()
 
 
 
